@@ -1,5 +1,4 @@
 """Module to communicate with the SecuritySpy API."""
-from aiohttp import client_exceptions
 import asyncio
 import json as pjson
 import logging
@@ -8,16 +7,15 @@ from base64 import b64encode
 
 import aiohttp
 import xmltodict
+from aiohttp import client_exceptions
 
 from pysecspy.const import (
     CAMERA_MESSAGES,
     DEVICE_UPDATE_INTERVAL_SECONDS,
     EVENT_MESSAGES,
-    RECORDING_MODE_LIST,
     RECORDING_TYPE_ACTION,
     RECORDING_TYPE_CONTINUOUS,
     RECORDING_TYPE_MOTION,
-    RECORDING_TYPE_OFF,
     SERVER_ID,
     SERVER_NAME,
     WEBSOCKET_CHECK_INTERVAL_SECONDS,
@@ -91,10 +89,6 @@ class SecSpyServer:
         current_time = time.time()
         device_update = False
         if (
-            # not self.ws_connection
-            # and force_camera_update
-            # or (current_time - DEVICE_UPDATE_INTERVAL_SECONDS)
-            # > self._last_device_update_time
             force_camera_update
             or (current_time - DEVICE_UPDATE_INTERVAL_SECONDS)
             > self._last_device_update_time
@@ -106,7 +100,9 @@ class SecSpyServer:
         else:
             _LOGGER.debug("Skipping device update")
 
-        if (current_time - WEBSOCKET_CHECK_INTERVAL_SECONDS) > self._last_websocket_check:
+        if (
+            current_time - WEBSOCKET_CHECK_INTERVAL_SECONDS
+        ) > self._last_websocket_check:
             _LOGGER.debug("Checking websocket")
             self._last_websocket_check = current_time
             await self.async_connect_ws()
@@ -136,7 +132,6 @@ class SecSpyServer:
         await self.ws_connection.wait_for_close()
         await self.ws_session.close()
 
-
     async def _get_device_list(self, include_events) -> None:
         """Get a list of devices connected to the NVR."""
 
@@ -177,12 +172,17 @@ class SecSpyServer:
         json_raw = xmltodict.parse(data)
         json_response = pjson.loads(pjson.dumps(json_raw))
         nvr = json_response["system"]["server"]
+        sched_preset = json_response["system"]["schedulepresetlist"]
+        presets = []
+        for preset in sched_preset["schedulepreset"]:
+            presets.append(preset)
 
         return {
             SERVER_NAME: nvr["server-name"],
             "server_version": nvr["version"],
             SERVER_ID: nvr["uuid"],
             "server_ip_address": nvr["ip1"],
+            "schedule_presets": presets,
         }
 
     async def get_unique_id(self) -> None:
@@ -210,21 +210,59 @@ class SecSpyServer:
             )
         return await response.read()
 
-    async def set_camera_recording(self, camera_id: str, mode: str, value: bool) -> bool:
-        """Sets the camera recoding mode .
-        Valid inputs for mode: off, on_motion, continuous. Valid input for value is True or False
+    async def get_latest_motion_recording(self, camera_id: str) -> bytes:
+        """ Returns the latest motion recording file. """
+
+        # Get the latest file name
+        file_uri = f"{self._base_url}/download?cameraNum={camera_id}&mcFilesCheck=1&ageText=1&results=1&format=xml&auth={self._token}"
+        response = await self.req.get(
+            file_uri,
+            headers=self.headers,
+            ssl=False,
+        )
+        if response.status != 200:
+            raise RequestError(
+                f"Fetching Recording files failed: {response.status} - Reason: {response.reason}"
+            )
+        json_raw = xmltodict.parse(await response.read())
+        json_response = pjson.loads(pjson.dumps(json_raw))
+        download_url = json_response["feed"]["entry"]["link"]["@href"]
+
+        # Retrieve the file
+        video_uri = f"{self._base_url}/{download_url}?auth={self._token}"
+        _LOGGER.debug("VIDEO URI: %s", video_uri)
+
+        response = await self.req.get(
+            video_uri,
+            headers=self.headers,
+            ssl=False,
+        )
+        if response.status != 200:
+            raise RequestError(
+                f"Fetching Video Recording failed: {response.status} - Reason: {response.reason}"
+            )
+        return await response.read()
+
+    async def set_arm_mode(
+        self, camera_id: str, mode: str, enabled: bool
+    ) -> bool:
+        """Sets the camera arming mode .
+        Valid inputs for mode: action, on_motion, continuous. Valid input for value is True or False
         """
 
-        schedule = 1 if value else 0
+        schedule = 1 if enabled else 0
 
         if mode in RECORDING_TYPE_ACTION:
             rec_mode = "A"
+            json_id = "recording_mode_a"
 
         if mode in RECORDING_TYPE_MOTION:
             rec_mode = "M"
+            json_id = "recording_mode_m"
 
         if mode in RECORDING_TYPE_CONTINUOUS:
             rec_mode = "C"
+            json_id = "recording_mode_c"
 
         cam_uri = f"{self._base_url}/setSchedule?cameraNum={camera_id}&schedule={schedule}&override=0&mode={rec_mode}&auth={self._token}"
 
@@ -235,10 +273,32 @@ class SecSpyServer:
         )
         if response.status != 200:
             raise RequestError(
-                f"Setting Recording mode failed: {response.status} - Reason: {response.reason}"
+                f"Setting Arming mode failed: {response.status} - Reason: {response.reason}"
+            )
+
+        self._processed_data[camera_id][json_id] = enabled
+        return True
+
+    async def enable_schedule_preset(
+        self, schedule_id: str
+    ) -> bool:
+        """Enables a schedule preset.
+        Valid inputs for schedule_id is a valid preset id
+        Format: setPreset?id=X
+        """
+
+        cam_uri = f"{self._base_url}/setPreset?id={schedule_id}&auth={self._token}"
+
+        response = await self.req.get(
+            cam_uri,
+            headers=self.headers,
+            ssl=False,
+        )
+        if response.status != 200:
+            raise RequestError(
+                f"Setting Schedule Preset failed: {response.status} - Reason: {response.reason}"
             )
 
-        self._processed_data[camera_id]["recording_mode"] = mode
         return True
 
     def _process_cameras_json(self, json_response, server_id, include_events):
@@ -270,7 +330,9 @@ class SecSpyServer:
     async def _setup_streamreader(self):
         """Setup the Event Websocket."""
         url = f"{self._base_url}/eventStream?version=3&format=multipart&auth={self._token}"
-        timeout = aiohttp.ClientTimeout(total=None, connect=None, sock_connect=None, sock_read=None)
+        timeout = aiohttp.ClientTimeout(
+            total=None, connect=None, sock_connect=None, sock_read=None
+        )
         if not self.ws_session:
             self.ws_session = aiohttp.ClientSession(timeout=timeout)
         _LOGGER.debug("Receiving from: %s", url)
@@ -290,31 +352,12 @@ class SecSpyServer:
                         )
                         return
                 await asyncio.sleep(0)
-        except client_exceptions.ClientConnectionError as e:
+        except client_exceptions.ClientConnectionError:
             return
         finally:
             _LOGGER.debug("stream disconnected")
             self.ws_connection = None
 
-        # try:
-        #     async with self.ws_session.request("get", url) as self.ws_connection:
-        #         try:
-        #             async for msg in self.ws_connection.content:
-        #                 data = msg.decode("UTF-8").strip()
-        #                 try:
-        #                     if data[:14].isnumeric():
-        #                         self._process_ws_message(data)
-        #                 except Exception as err:
-        #                     _LOGGER.exception(
-        #                         "Error processing stream message. Error: %s", err
-        #                     )
-        #                     return
-        #         except client_exceptions.ClientConnectionError as e:
-        #             return
-        # finally:
-        #     _LOGGER.debug("stream disconnected")
-        #     self.ws_connection = None
-
     def subscribe_websocket(self, ws_callback):
         """Subscribe to websocket events.
 
@@ -409,7 +452,6 @@ class SecSpyServer:
                     "type": "motion",
                     "end": action_array[0],
                     "camera": action_array[2],
-                    "file_name": action_array[4],
                     "isMotionDetected": False,
                 }
                 action_json = {
@@ -464,7 +506,10 @@ class SecSpyServer:
             return
         _LOGGER.debug("Processed camera: %s", processed_camera)
 
-        if processed_camera["recording_mode"] == RECORDING_TYPE_OFF:
+        if (
+            not processed_camera["recording_mode_m"]
+            or not processed_camera["recording_mode_c"]
+        ):
             processed_event = camera_event_from_ws_frames(
                 self._device_state_machine, action_json, data_json
             )
